@@ -83,11 +83,11 @@ extern void  cb_view();
 extern void  cb_viewopt(Widget);
 extern void  cb_vperm(Widget);
 extern void  cb_vrename(Widget);
-extern void  dojerror(unsigned, BtjobRef);
+extern void  qdojerror(unsigned, BtjobRef);
 extern void  getitemsel(Widget, int, int, char *, char **(*)(), void (*)());
 extern void  msg_error();
 extern void  vselect(Widget, int, XmListCallbackStruct *);
-extern void  wjimsg(const unsigned, CBtjobRef);
+extern void  qwjimsg(const unsigned, CBtjobRef);
 extern void  wjmsg(const unsigned, const ULONG);
 extern int  chk_okcidel(const int);
 extern char **gen_rvars(char *);
